@@ -28,13 +28,15 @@ import subprocess
 import sys
 from tempfile import TemporaryDirectory
 import uuid
+import warnings
 
 import numpy as np
+from PIL import Image
 
 import matplotlib as mpl
 from matplotlib._animation_data import (
     DISPLAY_TEMPLATE, INCLUDED_FRAMES, JS_INCLUDE, STYLE_INCLUDE)
-from matplotlib import cbook
+from matplotlib import _api, cbook
 
 
 _log = logging.getLogger(__name__)
@@ -99,10 +101,6 @@ class MovieWriterRegistry:
     def __init__(self):
         self._registered = dict()
 
-    @cbook.deprecated("3.2")
-    def set_dirty(self):
-        """Sets a flag to re-setup the writers."""
-
     def register(self, name):
         """
         Decorator for registering a class under a name.
@@ -118,19 +116,6 @@ class MovieWriterRegistry:
             return writer_cls
         return wrapper
 
-    @cbook.deprecated("3.2")
-    def ensure_not_dirty(self):
-        """If dirty, reasks the writers if they are available"""
-
-    @cbook.deprecated("3.2")
-    def reset_available_writers(self):
-        """Reset the available state of all registered writers"""
-
-    @cbook.deprecated("3.2")
-    @property
-    def avail(self):
-        return {name: self._registered[name] for name in self.list()}
-
     def is_available(self, name):
         """
         Check if given writer is available by name.
@@ -276,8 +261,12 @@ class MovieWriter(AbstractMovieWriter):
     # stored.  Third-party writers cannot meaningfully set these as they cannot
     # extend rcParams with new keys.
 
-    exec_key = cbook._deprecate_privatize_attribute("3.3")
-    args_key = cbook._deprecate_privatize_attribute("3.3")
+    exec_key = _api.deprecate_privatize_attribute("3.3")
+    args_key = _api.deprecate_privatize_attribute("3.3")
+
+    # Pipe-based writers only support RGBA, but file-based ones support more
+    # formats.
+    supported_formats = ["rgba"]
 
     def __init__(self, fps=5, codec=None, bitrate=None, extra_args=None,
                  metadata=None):
@@ -296,7 +285,7 @@ class MovieWriter(AbstractMovieWriter):
             Extra command-line arguments passed to the underlying movie
             encoder.  The default, None, means to use
             :rc:`animation.[name-of-encoder]_args` for the builtin writers.
-        metadata : Dict[str, str], default: {}
+        metadata : dict[str, str], default: {}
             A dictionary of keys and values for metadata to include in the
             output file. Some keys that may be of use include:
             title, artist, genre, subject, copyright, srcform, comment.
@@ -312,8 +301,7 @@ class MovieWriter(AbstractMovieWriter):
 
         super().__init__(fps=fps, metadata=metadata, codec=codec,
                          bitrate=bitrate)
-
-        self.frame_format = 'rgba'
+        self.frame_format = self.supported_formats[0]
         self.extra_args = extra_args
 
     def _adjust_frame_size(self):
@@ -351,7 +339,12 @@ class MovieWriter(AbstractMovieWriter):
 
     def finish(self):
         """Finish any processing for writing the movie."""
-        self.cleanup()
+        overridden_cleanup = _api.deprecate_method_override(
+            __class__.cleanup, self, since="3.4", alternative="finish()")
+        if overridden_cleanup is not None:
+            overridden_cleanup()
+        else:
+            self._cleanup()  # Inline _cleanup() once cleanup() is removed.
 
     def grab_frame(self, **savefig_kwargs):
         # docstring inherited
@@ -360,21 +353,16 @@ class MovieWriter(AbstractMovieWriter):
         # All frames must have the same size to save the movie correctly.
         self.fig.set_size_inches(self._w, self._h)
         # Save the figure data to the sink, using the frame format and dpi.
-        self.fig.savefig(self._frame_sink(), format=self.frame_format,
+        self.fig.savefig(self._proc.stdin, format=self.frame_format,
                          dpi=self.dpi, **savefig_kwargs)
 
-    def _frame_sink(self):
-        """Return the place to which frames should be written."""
-        return self._proc.stdin
-
     def _args(self):
         """Assemble list of encoder-specific command-line arguments."""
         return NotImplementedError("args needs to be implemented by subclass.")
 
-    def cleanup(self):
+    def _cleanup(self):  # Inline to finish() once cleanup() is removed.
         """Clean-up and collect the process used to write the movie file."""
         out, err = self._proc.communicate()
-        self._frame_sink().close()
         # Use the encoding/errors that universal_newlines would use.
         out = TextIOWrapper(BytesIO(out)).read()
         err = TextIOWrapper(BytesIO(err)).read()
@@ -390,6 +378,10 @@ class MovieWriter(AbstractMovieWriter):
             raise subprocess.CalledProcessError(
                 self._proc.returncode, self._proc.args, out, err)
 
+    @_api.deprecated("3.4")
+    def cleanup(self):
+        self._cleanup()
+
     @classmethod
     def bin_path(cls):
         """
@@ -412,10 +404,10 @@ class FileMovieWriter(MovieWriter):
     This must be sub-classed to be useful.
     """
     def __init__(self, *args, **kwargs):
-        MovieWriter.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.frame_format = mpl.rcParams['animation.frame_format']
 
-    @cbook._delete_parameter("3.3", "clear_temp")
+    @_api.delete_parameter("3.3", "clear_temp")
     def setup(self, fig, outfile, dpi=None, frame_prefix=None,
               clear_temp=True):
         """
@@ -427,10 +419,9 @@ class FileMovieWriter(MovieWriter):
             The figure to grab the rendered frames from.
         outfile : str
             The filename of the resulting movie file.
-        dpi : float, optional
+        dpi : float, default: ``fig.dpi``
             The dpi of the output file. This, with the figure size,
             controls the size in pixels of the resulting movie file.
-            Default is ``fig.dpi``.
         frame_prefix : str, optional
             The filename prefix to use for temporary files.  If None (the
             default), files are written to a temporary directory which is
@@ -458,7 +449,11 @@ class FileMovieWriter(MovieWriter):
         self._temp_paths = list()
         self.fname_format_str = '%s%%07d.%s'
 
-    @cbook.deprecated("3.3")
+    def __del__(self):
+        if self._tmpdir:
+            self._tmpdir.cleanup()
+
+    @_api.deprecated("3.3")
     @property
     def clear_temp(self):
         return self._clear_temp
@@ -480,6 +475,10 @@ class FileMovieWriter(MovieWriter):
         if frame_format in self.supported_formats:
             self._frame_format = frame_format
         else:
+            _api.warn_external(
+                f"Ignoring file format {frame_format!r} which is not "
+                f"supported by {type(self).__name__}; using "
+                f"{self.supported_formats[0]} instead.")
             self._frame_format = self.supported_formats[0]
 
     def _base_temp_name(self):
@@ -487,39 +486,27 @@ class FileMovieWriter(MovieWriter):
         # for extension and the prefix.
         return self.fname_format_str % (self.temp_prefix, self.frame_format)
 
-    def _frame_sink(self):
-        # Creates a filename for saving using the basename and the current
-        # counter.
-        path = Path(self._base_temp_name() % self._frame_counter)
-
-        # Save the filename so we can delete it later if necessary
-        self._temp_paths.append(path)
-        _log.debug('FileMovieWriter.frame_sink: saving frame %d to path=%s',
-                   self._frame_counter, path)
-        self._frame_counter += 1  # Ensures each created name is 'unique'
-
-        # This file returned here will be closed once it's used by savefig()
-        # because it will no longer be referenced and will be gc-ed.
-        return open(path, 'wb')
-
     def grab_frame(self, **savefig_kwargs):
         # docstring inherited
         # Overloaded to explicitly close temp file.
-        _log.debug('MovieWriter.grab_frame: Grabbing frame.')
-        # Tell the figure to save its data to the sink, using the
-        # frame format and dpi.
-        with self._frame_sink() as myframesink:
-            self.fig.savefig(myframesink, format=self.frame_format,
-                             dpi=self.dpi, **savefig_kwargs)
+        # Creates a filename for saving using basename and counter.
+        path = Path(self._base_temp_name() % self._frame_counter)
+        self._temp_paths.append(path)  # Record the filename for later cleanup.
+        self._frame_counter += 1  # Ensures each created name is unique.
+        _log.debug('FileMovieWriter.grab_frame: Grabbing frame %d to path=%s',
+                   self._frame_counter, path)
+        with open(path, 'wb') as sink:  # Save figure to the sink.
+            self.fig.savefig(sink, format=self.frame_format, dpi=self.dpi,
+                             **savefig_kwargs)
 
     def finish(self):
         # Call run here now that all frame grabbing is done. All temp files
         # are available to be assembled.
         self._run()
-        MovieWriter.finish(self)  # Will call clean-up
+        super().finish()  # Will call clean-up
 
-    def cleanup(self):
-        MovieWriter.cleanup(self)
+    def _cleanup(self):  # Inline to finish() once cleanup() is removed.
+        super()._cleanup()
         if self._tmpdir:
             _log.debug('MovieWriter: clearing temporary path=%s', self._tmpdir)
             self._tmpdir.cleanup()
@@ -542,7 +529,6 @@ class PillowWriter(AbstractMovieWriter):
         self._frames = []
 
     def grab_frame(self, **savefig_kwargs):
-        from PIL import Image
         buf = BytesIO()
         self.fig.savefig(
             buf, **{**savefig_kwargs, "format": "rgba", "dpi": self.dpi})
@@ -642,19 +628,32 @@ class FFMpegFileWriter(FFMpegBase, FileMovieWriter):
     Frames are written to temporary files on disk and then stitched
     together at the end.
     """
-    supported_formats = ['png', 'jpeg', 'ppm', 'tiff', 'sgi', 'bmp',
-                         'pbm', 'raw', 'rgba']
+    supported_formats = ['png', 'jpeg', 'tiff', 'raw', 'rgba']
 
     def _args(self):
         # Returns the command line parameters for subprocess to use
         # ffmpeg to create a movie using a collection of temp images
-        return [self.bin_path(), '-r', str(self.fps),
-                '-i', self._base_temp_name(),
-                '-vframes', str(self._frame_counter)] + self.output_args
+        args = []
+        # For raw frames, we need to explicitly tell ffmpeg the metadata.
+        if self.frame_format in {'raw', 'rgba'}:
+            args += [
+                '-f', 'image2', '-vcodec', 'rawvideo',
+                '-video_size', '%dx%d' % self.frame_size,
+                '-pixel_format', 'rgba',
+                '-framerate', str(self.fps),
+            ]
+        args += ['-r', str(self.fps), '-i', self._base_temp_name(),
+                 '-vframes', str(self._frame_counter)]
+        # Logging is quieted because subprocess.PIPE has limited buffer size.
+        # If you have a lot of frames in your animation and set logging to
+        # DEBUG, you will have a buffer overrun.
+        if _log.getEffectiveLevel() > logging.DEBUG:
+            args += ['-loglevel', 'error']
+        return [self.bin_path(), *args, *self.output_args]
 
 
 # Base class of avconv information.  AVConv has identical arguments to FFMpeg.
-@cbook.deprecated('3.3')
+@_api.deprecated('3.3')
 class AVConvBase(FFMpegBase):
     """
     Mixin class for avconv output.
@@ -757,15 +756,17 @@ class ImageMagickFileWriter(ImageMagickBase, FileMovieWriter):
 
     Frames are written to temporary files on disk and then stitched
     together at the end.
-
     """
 
-    supported_formats = ['png', 'jpeg', 'ppm', 'tiff', 'sgi', 'bmp',
-                         'pbm', 'raw', 'rgba']
+    supported_formats = ['png', 'jpeg', 'tiff', 'raw', 'rgba']
 
     def _args(self):
-        return ([self.bin_path(), '-delay', str(self.delay), '-loop', '0',
-                 '%s*.%s' % (self.temp_prefix, self.frame_format)]
+        # Force format: ImageMagick does not recognize 'raw'.
+        fmt = 'rgba:' if self.frame_format == 'raw' else ''
+        return ([self.bin_path(),
+                 '-size', '%ix%i' % self.frame_size, '-depth', '8',
+                 '-delay', str(self.delay), '-loop', '0',
+                 '%s%s*.%s' % (fmt, self.temp_prefix, self.frame_format)]
                 + self.output_args)
 
 
@@ -780,6 +781,9 @@ def _included_frames(paths, frame_format):
 
 def _embedded_frames(frame_list, frame_format):
     """frame_list should be a list of base64-encoded png files"""
+    if frame_format == 'svg':
+        # Fix MIME type for svg
+        frame_format = 'svg+xml'
     template = '  frames[{0}] = "data:image/{1};base64,{2}"\n'
     return "\n" + "".join(
         template.format(i, frame_format, frame_data.replace('\n', '\\\n'))
@@ -791,11 +795,8 @@ class HTMLWriter(FileMovieWriter):
     """Writer for JavaScript-based HTML movies."""
 
     supported_formats = ['png', 'jpeg', 'tiff', 'svg']
-
-    @cbook.deprecated("3.3")
-    @property
-    def args_key(self):
-        return 'animation.html_args'
+    args_key = _api.deprecated("3.3")(property(
+        lambda self: 'animation.html_args'))
 
     @classmethod
     def isAvailable(cls):
@@ -810,8 +811,8 @@ class HTMLWriter(FileMovieWriter):
         extra_args = ()  # Don't lookup nonexistent rcParam[args_key].
         self.embed_frames = embed_frames
         self.default_mode = default_mode.lower()
-        cbook._check_in_list(['loop', 'once', 'reflect'],
-                             default_mode=self.default_mode)
+        _api.check_in_list(['loop', 'once', 'reflect'],
+                           default_mode=self.default_mode)
 
         # Save embed limit, which is given in MB
         if embed_limit is None:
@@ -825,8 +826,7 @@ class HTMLWriter(FileMovieWriter):
 
     def setup(self, fig, outfile, dpi, frame_dir=None):
         outfile = Path(outfile)
-        cbook._check_in_list(['.html', '.htm'],
-                             outfile_extension=outfile.suffix)
+        _api.check_in_list(['.html', '.htm'], outfile_extension=outfile.suffix)
 
         self._saved_frames = []
         self._total_bytes = 0
@@ -915,6 +915,12 @@ class Animation:
     This class is not usable as is, and should be subclassed to provide needed
     behavior.
 
+    .. note::
+
+        You must store the created Animation in a variable that lives as long
+        as the animation should run. Otherwise, the Animation object will be
+        garbage-collected and the animation stops.
+
     Parameters
     ----------
     fig : `~matplotlib.figure.Figure`
@@ -936,6 +942,8 @@ class Animation:
     """
 
     def __init__(self, fig, event_source=None, blit=False):
+        self._draw_was_started = False
+
         self._fig = fig
         # Disables blitting for backends that don't support it.  This
         # allows users to request it if available, but still have a
@@ -960,6 +968,14 @@ class Animation:
         if self._blit:
             self._setup_blit()
 
+    def __del__(self):
+        if not getattr(self, '_draw_was_started', True):
+            warnings.warn(
+                'Animation was deleted without rendering anything. This is '
+                'most likely unintended. To prevent deletion, assign the '
+                'Animation to a variable that exists for as long as you need '
+                'the Animation.')
+
     def _start(self, *args):
         """
         Starts interactive animation. Adds the draw frame command to the GUI
@@ -1024,7 +1040,7 @@ class Animation:
             encoder.  The default, None, means to use
             :rc:`animation.[name-of-encoder]_args` for the builtin writers.
 
-        metadata : Dict[str, str], default {}
+        metadata : dict[str, str], default: {}
             Dictionary of keys and values for metadata to include in
             the output file. Some keys that may be of use include:
             title, artist, genre, subject, copyright, srcform, comment.
@@ -1195,7 +1211,7 @@ class Animation:
     def _init_draw(self):
         # Initial draw to clear the frame. Also used by the blitting code
         # when a clean base is required.
-        pass
+        self._draw_was_started = True
 
     def _pre_draw(self, framedata, blit):
         # Perform any cleaning or whatnot before the drawing of the frame.
@@ -1396,6 +1412,20 @@ class Animation:
         elif fmt == 'jshtml':
             return self.to_jshtml()
 
+    def pause(self):
+        """Pause the animation."""
+        self.event_source.stop()
+        if self._blit:
+            for artist in self._drawn_artists:
+                artist.set_animated(False)
+
+    def resume(self):
+        """Resume the animation."""
+        self.event_source.start()
+        if self._blit:
+            for artist in self._drawn_artists:
+                artist.set_animated(True)
+
 
 class TimedAnimation(Animation):
     """
@@ -1403,6 +1433,12 @@ class TimedAnimation(Animation):
 
     A new frame is drawn every *interval* milliseconds.
 
+    .. note::
+
+        You must store the created Animation in a variable that lives as long
+        as the animation should run. Otherwise, the Animation object will be
+        garbage-collected and the animation stops.
+
     Parameters
     ----------
     fig : `~matplotlib.figure.Figure`
@@ -1428,8 +1464,7 @@ class TimedAnimation(Animation):
         # sharing timers between animation objects for syncing animations.
         if event_source is None:
             event_source = fig.canvas.new_timer(interval=self._interval)
-        Animation.__init__(self, fig, event_source=event_source,
-                           *args, **kwargs)
+        super().__init__(fig, event_source=event_source, *args, **kwargs)
 
     def _step(self, *args):
         """Handler for getting events."""
@@ -1439,31 +1474,16 @@ class TimedAnimation(Animation):
         # _repeat_delay is set, change the event_source's interval to our loop
         # delay and set the callback to one which will then set the interval
         # back.
-        still_going = Animation._step(self, *args)
+        still_going = super()._step(*args)
         if not still_going and self.repeat:
             self._init_draw()
             self.frame_seq = self.new_frame_seq()
-            self.event_source.remove_callback(self._step)
-            self.event_source.add_callback(self._loop_delay)
             self.event_source.interval = self._repeat_delay
             return True
         else:
+            self.event_source.interval = self._interval
             return still_going
 
-    def _stop(self, *args):
-        # If we stop in the middle of a loop delay (which is relatively likely
-        # given the potential pause here), remove the loop_delay callback as
-        # well.
-        self.event_source.remove_callback(self._loop_delay)
-        Animation._stop(self)
-
-    def _loop_delay(self, *args):
-        # Reset the interval and change callbacks after the delay.
-        self.event_source.remove_callback(self._loop_delay)
-        self.event_source.interval = self._interval
-        self.event_source.add_callback(self._step)
-        Animation._step(self)
-
 
 class ArtistAnimation(TimedAnimation):
     """
@@ -1472,13 +1492,19 @@ class ArtistAnimation(TimedAnimation):
     Before creating an instance, all plotting should have taken place
     and the relevant artists saved.
 
+    .. note::
+
+        You must store the created Animation in a variable that lives as long
+        as the animation should run. Otherwise, the Animation object will be
+        garbage-collected and the animation stops.
+
     Parameters
     ----------
     fig : `~matplotlib.figure.Figure`
         The figure object used to get needed events, such as draw or resize.
     artists : list
-        Each list entry is a collection of artists that are made visible on
-        the corresponding frame.  Other artists are made invisible.
+        Each list entry is a collection of `.Artist` objects that are made
+        visible on the corresponding frame.  Other artists are made invisible.
     interval : int, default: 200
         Delay between frames in milliseconds.
     repeat_delay : int, default: 0
@@ -1497,9 +1523,10 @@ class ArtistAnimation(TimedAnimation):
         # Use the list of artists as the framedata, which will be iterated
         # over by the machinery.
         self._framedata = artists
-        TimedAnimation.__init__(self, fig, *args, **kwargs)
+        super().__init__(fig, *args, **kwargs)
 
     def _init_draw(self):
+        super()._init_draw()
         # Make all the artists involved in *any* frame invisible
         figs = set()
         for f in self.new_frame_seq():
@@ -1538,6 +1565,12 @@ class FuncAnimation(TimedAnimation):
     """
     Makes an animation by repeatedly calling a function *func*.
 
+    .. note::
+
+        You must store the created Animation in a variable that lives as long
+        as the animation should run. Otherwise, the Animation object will be
+        garbage-collected and the animation stops.
+
     Parameters
     ----------
     fig : `~matplotlib.figure.Figure`
@@ -1642,10 +1675,10 @@ class FuncAnimation(TimedAnimation):
             self._iter_gen = frames
         elif np.iterable(frames):
             if kwargs.get('repeat', True):
+                self._tee_from = frames
                 def iter_frames(frames=frames):
-                    while True:
-                        this, frames = itertools.tee(frames, 2)
-                        yield from this
+                    this, self._tee_from = itertools.tee(self._tee_from, 2)
+                    yield from this
                 self._iter_gen = iter_frames
             else:
                 self._iter_gen = lambda: iter(frames)
@@ -1669,7 +1702,7 @@ class FuncAnimation(TimedAnimation):
         # Needs to be initialized so the draw functions work without checking
         self._save_seq = []
 
-        TimedAnimation.__init__(self, fig, **kwargs)
+        super().__init__(fig, **kwargs)
 
         # Need to reset the saved seq, since right now it will contain data
         # for a single frame from init, which is not what we want.
@@ -1702,7 +1735,7 @@ class FuncAnimation(TimedAnimation):
                     except StopIteration:
                         pass
                     else:
-                        cbook.warn_deprecated(
+                        _api.warn_deprecated(
                             "2.2", message="FuncAnimation.save has truncated "
                             "your animation to 100 frames.  In the future, no "
                             "such truncation will occur; please pass "
@@ -1711,6 +1744,7 @@ class FuncAnimation(TimedAnimation):
                 return gen()
 
     def _init_draw(self):
+        super()._init_draw()
         # Initialize the drawing either using the given init_func or by
         # calling the draw function with the first item of the frame sequence.
         # For blitting, the init_func should return a sequence of modified
@@ -1740,10 +1774,22 @@ class FuncAnimation(TimedAnimation):
         # Call the func with framedata and args. If blitting is desired,
         # func needs to return a sequence of any artists that were modified.
         self._drawn_artists = self._func(framedata, *self._args)
+
         if self._blit:
-            if self._drawn_artists is None:
-                raise RuntimeError('The animation function must return a '
-                                   'sequence of Artist objects.')
+
+            err = RuntimeError('The animation function must return a sequence '
+                               'of Artist objects.')
+            try:
+                # check if a sequence
+                iter(self._drawn_artists)
+            except TypeError:
+                raise err from None
+
+            # check each item if is artist
+            for i in self._drawn_artists:
+                if not isinstance(i, mpl.artist.Artist):
+                    raise err
+
             self._drawn_artists = sorted(self._drawn_artists,
                                          key=lambda x: x.get_zorder())
 
